@@ -25,6 +25,7 @@
  *	Kristian Høgsberg (krh@redhat.com)
  *	Adam Jackson (ajax@redhat.com)
  *	Peter Hutterer (peter@cs.unisa.edu.au)
+ *	Oliver McFadden (oliver.mcfadden@nokia.com)
  */
 
 #ifndef EVDEV_H
@@ -35,10 +36,7 @@
 
 #include <xf86Xinput.h>
 #include <xf86_OSproc.h>
-
-#ifdef XKB
 #include <xkbstr.h>
-#endif
 
 #ifndef EV_CNT /* linux 2.4 kernels and earlier lack _CNT defines */
 #define EV_CNT (EV_MAX+1)
@@ -57,11 +55,28 @@
 #endif
 
 #define EVDEV_MAXBUTTONS 32
+#define EVDEV_MAXQUEUE 32
 
 #if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 3
 #define HAVE_PROPERTIES 1
 #endif
 
+#ifndef MAX_VALUATORS
+#define MAX_VALUATORS 36
+#endif
+
+
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 5
+typedef struct {
+    char *rules;
+    char *model;
+    char *layout;
+    char *variant;
+    char *options;
+} XkbRMLVOSet;
+#endif
+
+
 #define LONG_BITS (sizeof(long) * 8)
 
 /* Number of longs needed to hold the given number of bits */
@@ -74,29 +89,41 @@ typedef struct {
     int traveled_distance;
 } WheelAxis, *WheelAxisPtr;
 
+/* Event queue used to defer keyboard/button events until EV_SYN time. */
+typedef struct {
+    enum {
+        EV_QUEUE_KEY,	/* xf86PostKeyboardEvent() */
+        EV_QUEUE_BTN,	/* xf86PostButtonEvent() */
+    } type;
+    int key;		/* May be either a key code or button number. */
+    int val;		/* State of the key/button; pressed or released. */
+} EventQueueRec, *EventQueuePtr;
+
 typedef struct {
     const char *device;
     int grabDevice;         /* grab the event device? */
-    int screen;
-    int min_x, min_y, max_x, max_y;
-    int abs_x, abs_y, old_x, old_y;
+
+    int num_vals;           /* number of valuators */
+    int axis_map[max(ABS_CNT, REL_CNT)]; /* Map evdev <axis> to index */
+    int vals[MAX_VALUATORS];
+    int old_vals[MAX_VALUATORS]; /* Translate absolute inputs to relative */
+
     int flags;
     int tool;
-    int buttons;            /* number of buttons */
+    int num_buttons;            /* number of buttons */
     BOOL swap_axes;
     BOOL invert_x;
     BOOL invert_y;
 
+    int delta[REL_CNT];
+    unsigned int abs, rel;
+
     /* XKB stuff has to be per-device rather than per-driver */
-    int noXkb;
-#ifdef XKB
-    char                    *xkb_rules;
-    char                    *xkb_model;
-    char                    *xkb_layout;
-    char                    *xkb_variant;
-    char                    *xkb_options;
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 5
     XkbComponentNamesRec    xkbnames;
 #endif
+    XkbRMLVOSet rmlvo;
+
     /* Middle mouse button emulation */
     struct {
         BOOL                enabled;
@@ -138,17 +165,30 @@ typedef struct {
 
     /* Cached info from device. */
     char name[1024];
-    long bitmask[NLONGS(EV_CNT)];
-    long key_bitmask[NLONGS(KEY_CNT)];
-    long rel_bitmask[NLONGS(REL_CNT)];
-    long abs_bitmask[NLONGS(ABS_CNT)];
-    long led_bitmask[NLONGS(LED_CNT)];
+    unsigned long bitmask[NLONGS(EV_CNT)];
+    unsigned long key_bitmask[NLONGS(KEY_CNT)];
+    unsigned long rel_bitmask[NLONGS(REL_CNT)];
+    unsigned long abs_bitmask[NLONGS(ABS_CNT)];
+    unsigned long led_bitmask[NLONGS(LED_CNT)];
     struct input_absinfo absinfo[ABS_CNT];
 
     /* minor/major number */
     dev_t min_maj;
+
+    /* Event queue used to defer keyboard/button events until EV_SYN time. */
+    int                     num_queue;
+    EventQueueRec           queue[EVDEV_MAXQUEUE];
 } EvdevRec, *EvdevPtr;
 
+/* Event posting functions */
+void EvdevQueueKbdEvent(InputInfoPtr pInfo, struct input_event *ev, int value);
+void EvdevQueueButtonEvent(InputInfoPtr pInfo, int button, int value);
+void EvdevPostButtonEvent(InputInfoPtr pInfo, int button, int value);
+void EvdevQueueButtonClicks(InputInfoPtr pInfo, int button, int count);
+void EvdevPostRelativeMotionEvents(InputInfoPtr pInfo, int *num_v, int *first_v,
+				   int v[MAX_VALUATORS]);
+void EvdevPostAbsoluteMotionEvents(InputInfoPtr pInfo, int *num_v, int *first_v,
+				   int v[MAX_VALUATORS]);
 unsigned int EvdevUtilButtonEventToButtonNumber(EvdevPtr pEvdev, int code);
 
 /* Middle Button emulation */
